@@ -22,7 +22,6 @@ require 'lib/common'
 require 'lib/messages'
 require 'interface/input'
 require 'lib/shadowget'
-require 'lib/searchcode'
 require 'lib/search'
 require 'lib/filter'
 require 'lib/output'
@@ -71,19 +70,23 @@ class GeoToad
     if @option['help']
       @uin.usage
       exit
-    end    
-    
-    if (! @option['clearCache']) && ((! @queryArg) || (! @option['user']) || (! @option['password']))
-      if (! @queryArg)
-        displayError "You forgot to specify a #{@queryType} search argument"
-      end
-      if (! @option['user']) ||  (! @option['password'])
-        displayError "You must specify a username and password to download coordinates from Geocaching.com"
-      end
+    end
+
+    if ! @option['clearCache'] && ! @queryArg
+      displayError "You forgot to specify a #{@queryType} search argument"
       @uin.usage
       exit
     end
 
+    if (! @option['user']) || (! @option['password'])
+      debug "No user/password option given, loading from config."
+      (@option['user'], @option['password']) = @uin.loadUserAndPasswordFromConfig()
+      if (! @option['user']) || (! @option['password'])
+        displayError "You must specify a username and password to download coordinates from Geocaching.com"
+        exit
+      end
+    end
+
     @formatType        = @option['format'] || 'gpx'
     @cacheExpiry       = @option['cacheExpiry'].to_i || 3
     @distanceMax       = @option['distanceMax'] || 10
@@ -112,12 +115,12 @@ class GeoToad
     puts version
     return version
   end
-    
+
   def versionCheck
     if $VERSION =~ /CURRENT/
       return nil
     end
-  
+
     url = "http://code.google.com/p/geotoad/wiki/CurrentVersion";
 
     debug "Checking for latest version of GeoToad from #{url}"
@@ -152,7 +155,7 @@ class GeoToad
     puts "* Cleared!"
     $CACHE_DIR = findCacheDir()
   end
-    
+
   ## Make the Initial Query ############################
   def downloadGeocacheList
     displayInfo "Your cache directory is " + $CACHE_DIR
@@ -173,7 +176,7 @@ class GeoToad
         puts "(constraining to #{@distanceMax} miles)"
         @queryTitle = @queryTitle + " (#{@distanceMax}mi. radius)"
         @defaultOutputFile = @defaultOutputFile + "-y" + @distanceMax.to_s
-        search.distance=@distanceMax.to_i
+        search.distance = @distanceMax
       else
         puts
       end
@@ -212,22 +215,21 @@ class GeoToad
     # Prepare for the manipulation
     @filtered = Filter.new(@combinedWaypoints)
 
-
     # This is where we do a little bit of cheating. In order to avoid downloading the
     # cache details for each cache to see if it's been visited, we do a search for the
     # users on the include or exclude list. We then populate @combinedWaypoints[wid]['visitors']
     # with our discovery.
 
     userLookups = Array.new
-    if (@option['userExclude'])
+    if @option['userExclude'] and not @option['userExclude'].empty?
       @queryTitle = @queryTitle + ", excluding caches done by " + @option['userExclude']
-      @defaultOutputFile = @defaultOutputFile + "-U=" + @option['userExclude']
+      @defaultOutputFile = @defaultOutputFile + "-E=" + @option['userExclude']
       userLookups = @option['userExclude'].split(':')
     end
 
-    if (@option['userInclude'])
+    if @option['userInclude'] and not @option['userInclude'].empty?
       @queryTitle = @queryTitle + ", excluding caches not done by " + @option['userInclude']
-      @defaultOutputFile = @defaultOutputFile + "-u=" + @option['userInclude']
+      @defaultOutputFile = @defaultOutputFile + "-e=" + @option['userInclude']
       userLookups = userLookups + @option['userInclude'].split(':')
     end
 
@@ -289,7 +291,7 @@ class GeoToad
 
     if @option['sizeMax']
       @queryTitle = @queryTitle + ", size #{@option['sizeMax']} or lower"
-      @defaultOutputFile = @defaultOutputFile + "-S" + @option['sizeMin'].to_s
+      @defaultOutputFile = @defaultOutputFile + "-S" + @option['sizeMax'].to_s
       @filtered.sizeMax(@option['sizeMax'])
     end
 
@@ -308,13 +310,13 @@ class GeoToad
 
     if @option['placeDateInclude']
       @queryTitle = @queryTitle + ", newer than #{@option['placeDateInclude']} days"
-      @defaultOutputFile = @defaultOutputFile + "-p=" + @option['placeDateInclude'].to_s
+      @defaultOutputFile = @defaultOutputFile + "-j=" + @option['placeDateInclude'].to_s
       @filtered.placeDateInclude(@option['placeDateInclude'].to_f)
     end
 
     if @option['placeDateExclude']
       @queryTitle = @queryTitle + ", over #{@option['placeDateExclude']} days old"
-      @defaultOutputFile = @defaultOutputFile + "-P=" + @option['placeDateExclude'].to_s
+      @defaultOutputFile = @defaultOutputFile + "-J=" + @option['placeDateExclude'].to_s
       @filtered.placeDateExclude(@option['placeDateExclude'].to_f)
     end
     debug "Filter running cycle 4, #{@filtered.totalWaypoints} caches left"
@@ -418,6 +420,10 @@ class GeoToad
 
       if status == 'subscriber-only'
         message = '(subscriber-only)'
+      elsif status == 'unpublished'
+        wpFiltered.delete(wid)
+        displayMessage "#{wid} is unpublished, skipping."
+        next
       elsif ! status or status == 'login-required'
         if (wpFiltered[wid]['warning'])
           debug "Could not parse page, but it had a warning, so I am not invalidating"
@@ -469,7 +475,7 @@ class GeoToad
     # caches with warnings we choose not to include.
     if ! @option['includeDisabled']
       displayMessage "Filtering out disabled caches"
-      @filtered.removeByElement('warning')
+      @filtered.removeByElement('disabled')
     end
 
     if @option['descKeyword']
@@ -486,7 +492,7 @@ class GeoToad
 
     if @option['funFactorMax']
       @queryTitle = @queryTitle + ", funFactor #{@option['funFactorMax']} or lower"
-      @defaultOutputFile = @defaultOutputFile + '-A' + @option['funFactorMax'].to_s
+      @defaultOutputFile = @defaultOutputFile + '-F' + @option['funFactorMax'].to_s
       @filtered.funFactorMax(@option['funFactorMax'].to_f)
     end
 
@@ -524,7 +530,7 @@ class GeoToad
     output = Output.new
     displayInfo "Output format selected is #{output.formatDesc(@formatType)} format"
     output.input(@filtered.waypoints)
-    output.formatType=@formatType
+    output.formatType = @formatType
     if (@option['waypointLength'])
       output.waypointLength=@option['waypointLength'].to_i
     end
@@ -532,17 +538,27 @@ class GeoToad
 
     # if we have selected the name of the output file, use it.
     # otherwise, take our invented name, sanitize it, and slap a file extension on it.
-    if @option['output'] && (@option['output'] !~ /\/$/)
-      outputFile = File.basename(@option['output'])
-    else
-      outputFile = @defaultOutputFile.gsub(/\W/, '_')
+    filename = @option['output']
+    displayInfo "Output filename: #{filename}"
+    outputFile = nil
+
+    if filename
+      filename.gsub!('\\', '/')
+      if filename and filename !~ /\/$/
+        outputFile = File.basename(filename)
+      end
+    end
+
+    if not outputFile
+      outputFile = @defaultOutputFile.gsub(/[^0-9A-Za-z\.-]/, '_')
       outputFile.gsub!(/_+/, '_')
       if outputFile.length > 220
         outputFile = outputFile[0..215] + "_etc"
       end
-      
+
       outputFile = outputFile + "." + output.formatExtension(@formatType)
     end
+    debug "Base output path: #{outputFile}"
 
     # prepend the current working directory. This is mostly done as a service to
     # users who just double click to launch GeoToad, and wonder where their output file went.
@@ -574,7 +590,7 @@ class GeoToad
     @queryTitle = @queryTitle + " (" + Time.now.strftime("%d%b%y %H:%M") + ")"
 
     # and do the dirty.
-    outputData = output.prepare(@queryTitle);
+    outputData = output.prepare(@queryTitle, @option['user']);
     output.commit(outputFile)
     displayMessage "Saved to #{outputFile}"
   end
@@ -599,7 +615,7 @@ while(1)
   if options['clearCache']
     cli.clearCacheDirectory()
   end
-  
+
   count = cli.downloadGeocacheList()
   if count < 1
     cli.displayError "No caches found in search, exiting early."
