@@ -1,4 +1,9 @@
-# $Id: input.rb 606 2010-08-08 04:09:06Z helixblue $
+# $Id: input.rb 656 2010-10-17 20:57:57Z helixblue $
+$LOAD_PATH << (File.dirname(__FILE__.gsub(/\\/, '/')) + '/' + '..')
+$LOAD_PATH << (File.dirname(__FILE__.gsub(/\\/, '/')) + '/' + '../lib')
+
+require 'country_state'
+
 
 class Input
   include Common
@@ -26,11 +31,14 @@ class Input
     end
 
     # File contains password, keep it safe..
-    f=File.open(@configFile, 'w', 0600)
+    f = File.open(@configFile, 'w', 0600)
+    @@optHash.each_key {|key|
+      if @@optHash[key].to_s.empty?
+        @@optHash.delete(key)
+      end
+    }
+
     f.puts @@optHash.to_yaml
-    #@@optHash.each_key { |option|
-    #    f.puts option + ": " + @@optHash[option].to_s
-    #}
     f.close
     debug "Saved configuration"
   end
@@ -38,7 +46,16 @@ class Input
   def loadConfig
     if File.exists?(@configFile)
       displayMessage "Loading configuration from #{@configFile}"
-      @@optHash = YAML::load( File.open(@configFile) )
+      return YAML::load(File.open(@configFile))
+    end
+  end
+
+  def loadUserAndPasswordFromConfig
+    data = loadConfig()
+    if data
+      return [data['user'], data['password']]
+    else
+      return [nil, nil]
     end
   end
 
@@ -92,7 +109,7 @@ class Input
       [ "--format",                    "-x",        GetoptLong::OPTIONAL_ARGUMENT ],
 
       [ "--distanceMax",                "-y",        GetoptLong::OPTIONAL_ARGUMENT ],
-      [ "--includeDisabled",          "-z",    GetoptLong::NO_ARGUMENT ]      
+      [ "--includeDisabled",          "-z",    GetoptLong::NO_ARGUMENT ]
     ) || usage
 
     # put the stupid crap in a hash. Much nicer to deal with.
@@ -113,9 +130,7 @@ class Input
       exit
     end
 
-
     @@optHash['queryArg'] = ARGV.shift
-
     # if there are still remaining arguments, error out. Usually missed quote marks.
     # We used to make assumptions about this, but it ended up being more confusing when
     # wrong.
@@ -132,9 +147,13 @@ class Input
 
   def interactive
     # pop up the menu
-    loadConfig
-    showMenu
-    saveConfig
+    loaded_config = loadConfig()
+    if loaded_config
+      @@optHash = loaded_config
+    end
+    showMenu()
+    saveConfig()
+
     if (@@optHash['outDir'])
       @@optHash['output']=@@optHash['outDir'] + "/"
     else
@@ -152,13 +171,18 @@ class Input
 
     # demonstrate a sample command line
     cmdline = "geotoad.rb"
+    hidden_opts = ['queryArg', 'outDir', 'outFile', 'user', 'password']
+
     @@optHash.keys.sort.each { |option|
-      if (option != 'queryArg') && (option != 'outDir') && (option != 'outFile') && @@optHash[option]
+      if ! @@optHash[option].to_s.empty? and ! hidden_opts.include?(option)
+        # This is the default query type
+        if option == 'queryType' and @@optHash[option] == 'location'
+          next
+        end
+
         if @@optHash[option] == 'X'
           cmdline = cmdline + " --#{option}"
-        elsif @@optHash[option] == ''
-          cmdline = cmdline
-        else
+        elsif not @@optHash[option].to_s.empty?
           # Omit the quotes if the argument is 'simple'
           if @@optHash[option].to_s =~ /^[\w\.:]+$/
             cmdline = cmdline + " --#{option}=#{@@optHash[option]}"
@@ -169,7 +193,11 @@ class Input
       end
 
     }
-    cmdline = cmdline + " \'" + @@optHash['queryArg'].to_s + '\''
+    if @@optHash['queryArg'].to_s =~ /^[\w\.:]+$/
+      cmdline = cmdline + " " + @@optHash['queryArg'].to_s
+    else
+      cmdline = cmdline + " \'" + @@optHash['queryArg'].to_s + '\''
+    end
     displayMessage "To use this query in the future, type:"
     displayMessage cmdline
     puts
@@ -186,7 +214,7 @@ class Input
 
     puts " -o [filename]          output file name (automatic otherwise)"
     puts " -x [format]            output format type, see list below"
-    puts " -q [zip|state|coord|country|user|wid]   query type (zip by default)"
+    puts " -q [location|user|wid]   query type (location by default)"
 
     puts " -d/-D [0.0-5.0]        difficulty minimum/maximum"
     puts " -t/-T [0.0-5.0]        terrain minimum/maximum"
@@ -235,7 +263,7 @@ class Input
     puts ""
     puts "::: EXAMPLES:"
     puts "  geotoad.rb -u helixblue -p password 27502"
-    puts "  geotoad.rb -u john -p password -d 3 -s helixblue -f vcf -o NC.vcf -q state \'North Carolina\'"
+    puts "  geotoad.rb -u john -p password -d 3 -s helixblue -f vcf -o NC.vcf \'North Carolina\'"
   end
 
   def showMenu
@@ -256,7 +284,6 @@ class Input
       puts "=============================================================================="
       printf(":::           %46.46s               :::\n", "// GeoToad #$VERSION Text User Interface //")
       puts "=============================================================================="
-
       printf("(1)  GC.com login [%-17.17s] | (2)  search type          [%-10.10s]\n", (@@optHash['user'] || 'REQUIRED'), @@optHash['queryType'])
       printf("(3)  %-12.12s [%-17.17s] | (4)  distance maximum            [%-3.3s]\n", @@optHash['queryType'], (@@optHash['queryArg'] || 'REQUIRED'), (@@optHash['distanceMax'] || 10))
       puts   "                                      |"
@@ -275,7 +302,7 @@ class Input
       printf("(16) cache not found by  [%-10.10s] | (17) cache owner isn't [%-13.13s]\n", @@optHash['userExclude'], @@optHash['ownerExclude'])
       printf("(18) cache found by      [%-10.10s] | (19) cache owner is    [%-13.13s]\n", @@optHash['userInclude'], @@optHash['ownerInclude'])
 
-      printf("(20) EasyName WP length         [%3.3s] | (21) include disabled caches [%1.1s] \n", @@optHash['waypointLength'] || '0', @@optHash['includeDisabled'])
+      printf("(20) EasyName WP length         [%3.3s] | (21) include disabled caches       [%1.1s] \n", @@optHash['waypointLength'] || '0', @@optHash['includeDisabled'])
       puts "- - - - - - - - - - - - - - - - - - - + - - - - - - - - - - - - - - - - - - -"
       printf("(22) output format       [%-10.10s]   (23) filename   [%-20.20s]\n", (@@optHash['format'] || 'gpx'), (@@optHash['outFile'] || 'automatic'))
       printf("(24) output directory    [%-51.51s]\n", (@@optHash['outDir'] || findOutputDir))
@@ -297,45 +324,42 @@ class Input
         @@optHash['password'] = ask("What is your Geocaching.com password?", 'NO_DEFAULT')
 
       when '2'
-        guess = nil
-        while not guess
-          type = ask("What type of search would you like to perform? (location, state, country, user, keyword [title only])", nil)
-          guess = guessQueryType(type)
-          if not type
-            puts "** You need to specify a search type."
-          elsif not guess
-            puts "** I could not guess what you meant by '#{type}''"
-          end
+        # TODO(helixblue): Add state searches back in once things settle down.
+        chosen = askFromList("What type of search would you like to perform:
+
+  1. Within distance of a location (landmark, city, postal code, coordinates) - DEFAULT
+  2. All caches found by a user
+  3. By title keyword
+  4. By coordinates
+  5. By waypoint ID
+
+", ['location', 'user', 'keyword', 'coord', 'wid'], 'location')
+
+        # Clear the query argument if the type has changed.
+        if @@optHash['queryType'] != chosen
+          @@optHash['queryArg'] = nil
         end
-        @@optHash['queryType'] = guess
+
+        @@optHash['queryType'] = chosen
 
       when '3'
-        if (@@optHash['queryType'] == 'location')
-          @@optHash['queryArg'] = ask('Type in an address, city/state/country, zip, or coordinates (uses Google Maps)', 'NO_DEFAULT')
-        end
+        case @@optHash['queryType']
+        when 'location'
+          @@optHash['queryArg'] = ask("Type in an address, city, state, postal code, or coordinates (uses Google Maps).\nMultiple locations may be separated by the | symbol\n\n", 'NO_DEFAULT')
 
-        if (@@optHash['queryType'] == 'zipcode')
-          @@optHash['queryArg'] = ask('Enter a list of zipcodes (seperated by commas)', 'NO_DEFAULT').gsub(/, */, ':')
-        end
+        when 'country'
+          @@optHash['queryArg'] = askCountry()
 
-        if (@@optHash['queryType'] == 'state')
-          @@optHash['queryArg'] = ask('Enter a list of states (seperated by commas)', 'NO_DEFAULT').gsub(/, */, ':')
-        end
+        when 'state'
+          @@optHash['queryArg'] = askState()
 
-        if (@@optHash['queryType'] == 'country')
-          @@optHash['queryArg'] = ask('Enter a list of countries (seperated by commas)', 'NO_DEFAULT').gsub(/, */, ':')
-        end
-
-
-        if (@@optHash['queryType'] == 'wid')
+        when 'wid'
           @@optHash['queryArg'] = ask('Enter a list of waypoint id\'s (seperated by commas)', 'NO_DEFAULT').gsub(/, */, ':')
-        end
 
-        if (@@optHash['queryType'] == 'user')
+        when 'user'
           @@optHash['queryArg'] = ask('Enter a list of users (seperated by commas)', 'NO_DEFAULT').gsub(/, */, ':')
-        end
 
-        if (@@optHash['queryType'] == 'coord')
+        when 'coord'
           puts "You will be asked to enter in a list of coordinates in the following format:"
           puts "N56 44.392 E015 52.780"
           puts ""
@@ -356,9 +380,8 @@ class Input
 
           query.gsub!(/:$/, '')
           @@optHash['queryArg'] = query
-        end
 
-        if (@@optHash['queryType'] == 'keyword')
+        when 'keyword'
           puts "Please enter a list of keywords, pressing enter after each one."
           puts "Press (q) when done."
 
@@ -381,7 +404,7 @@ class Input
 
 
       when '4'
-        @@optHash['distanceMax'] = askNumber("What is the maximum distance from your #{@@optHash['queryType']} that you would like to include geocaches from?", 10)
+        @@optHash['distanceMax'] = askNumber("How far away are you willing to search (miles)", 10)
 
       when '5'
         @@optHash['difficultyMin'] = askNumber('What is the minimum difficulty you would like? (0.0)', nil)
@@ -429,10 +452,10 @@ class Input
         @@optHash['foundDateInclude'] = askNumber('How many days ago is the maximum a geocache can be found in for your list? (any)', nil)
 
       when '14'
-        @@optHash['titleKeyword'] = ask('Only include geocaches with these keywords in their title (seperate by |)?', nil)
+        @@optHash['titleKeyword'] = ask('Filter caches by title keywords (negate using !, seperate multiple using |): ', nil)
 
       when '15'
-        @@optHash['descKeyword'] = ask('Only include geocaches with these keywords in their description (seperate by |)', nil)
+        @@optHash['descKeyword'] = ask('Filter caches by description keywords (negate using !, seperate multiple using |): ', nil)
 
       when '16'
         @@optHash['userExclude'] = ask('Filter out geocaches found by these people (seperate by commas)', '').gsub(/, */, ':')
@@ -564,12 +587,42 @@ class Input
     end
   end
 
+  def askCountry()
+    country = nil
+    c = CountryState.new()
+    while not country
+      try_country = ask("What country would you like to search for?", nil)
+      countries = c.findMatchingCountry(try_country)
+      if countries.length == 1
+        country = countries[0]
+      elsif countries.length > 1
+        i = 0
+        countries.each do |country|
+          i += 1
+          puts "  #{i}. #{country}"
+        end
+        country = askFromList("", countries, nil)
+      else
+        puts "No matches found. Try something else!"
+      end
+    end
+    return country
+  end
+
+  def askState()
+    puts "NOT YET IMPLEMENTED"
+  end
+
   def askFromList(string, choices, default)
-    # Ask for a floating point number.
+    # Ask for an item from a list. We accept either a number or word.
+
     try_again = 1
     while try_again
       begin
         answer = ask(string, default)
+        if answer.to_i > 0
+          return choices[answer.to_i - 1]
+        end
         if not answer
           return default
         end
@@ -577,6 +630,7 @@ class Input
         answers = answer.split(':')
         try_again = nil
         for try_answer in answers
+
           if ! choices.include?(try_answer)
             puts "** #{try_answer} is not valid! Try: #{choices.join(', ')}"
             try_again = 1
@@ -596,13 +650,19 @@ class Input
     when /loc/
       return 'location'
     when /zip/
-      return 'zipcode'
+      return 'location'
+    when /country/
+      return 'country'
+    when /state/
+      return 'state'
+    when /province/
+      return 'state'
     when /coo/
       return 'coord'
     when /stat/
-      return 'state'
+      return 'location'
     when /country/
-      return 'country'
+      return 'location'
     when /wid/
       return 'wid'
     when /waypoint/
