@@ -1,4 +1,5 @@
 # A library for automatic location searches, using the Google Geocoding API
+require 'csv'
 
 require 'common'
 require 'cgi'
@@ -6,12 +7,13 @@ require 'shadowget'
 
 MAPS_URL = 'http://maps.google.com/maps/geo'
 KEY = 'ABQIAAAAfYtme_pyDnFOuJLGZiXvPxRuVmV2GDBxlUzS4Tl93rTyZWZiOBRL-7BgtHIJc12HxIcS5teMAlIPzw'
+CACHE_SECONDS = 86400 * 180
 
 class GeoCode
   include Common
   include Messages
   
-  def lookup(location)
+  def lookup_location(location)
     data = get_url(create_url(location))
     code, accuracy, lat, lon = parse_data(data)
     if code == "200"
@@ -23,16 +25,28 @@ class GeoCode
     return return_data
   end
 
+  def lookup_coords(lat, lon)
+    coords = "#{lat},#{lon}"
+    debug "geocode looking up #{coords}"
+    data = get_url(create_url(coords))
+    code, accuracy, location = parse_data(data)
+    if code == "200"
+      return location
+    else
+      return "Unknown"
+    end
+  end
+
   def create_url(location)
     q = CGI.escape(location)
     url = "#{MAPS_URL}?q=#{q}&output=csv&oe=utf8&sensor=false&key=#{KEY}"
-    debug "url: #{url}"
+    debug "geocode url: #{url}"
     return url
   end
 
   def get_url(url)
     http = ShadowFetch.new(url)
-    http.localExpiry=86400 * 30
+    http.localExpiry = CACHE_SECONDS
     http.maxFailures = 5
     results = http.fetch
     debug "geocode data: #{results}"
@@ -40,9 +54,9 @@ class GeoCode
   end
 
   def decode_accuracy(value)
-    table = [nil, 'Country', 'Region (state, province)',
+    table = ['Continent', 'Country', 'Region (state, province)',
              'Sub-region (county, municipality)', 'Town', 'Post code', 'Street',
-             'Intersection', 'Address', 'Premise (building-name)']
+             'Intersection', 'Address', 'Area']
     if value
       desc = table[value.to_i]
     else
@@ -54,7 +68,7 @@ class GeoCode
 
   # Parse the CSV returned by http://code.google.com/apis/maps/documentation/geocoding/
   def parse_data(data)
-    return data.chomp.split(',')
+    return CSV.parse_line(data)
   end
 
 end
\ No newline at end of file
