@@ -1,4 +1,4 @@
-# $Id: output.rb 607 2010-08-08 05:29:28Z helixblue $
+# $Id: output.rb 659 2010-10-18 07:37:59Z helixblue $
 require 'cgi'
 require 'lib/templates'
 require 'lib/version'
@@ -77,6 +77,7 @@ class Output
   def initialize
     @output = Array.new
     @waypointLength = 0
+    @username = nil
   end
 
   def input(data)
@@ -183,8 +184,9 @@ class Output
   end
 
   ## sets up for the filtering process ################3
-  def prepare (title)
+  def prepare (title, username)
     @title = title
+    @username = username
 
     # if we are not actually generating the output, lets do it in a meta-fashion.
     debug "preparing for #{@outputType}"
@@ -275,14 +277,19 @@ class Output
     # CGI.escapeHTML will try to re-escape previously escaped entities.
     # Fix numerical entities such as Pateniemen l&amp;#228;mp&amp;#246;keskus
     text.gsub!(/&amp;([\#\d][\d]+;)/, "&\\1")
-    # XML only pre-defines the following named character entities:
-    text.gsub!('&amp;(amp;)', "&\\1")
 
-    # There is also [lg]t; &quot; &apos;, but they seem to be handled properly.
+    # XML only pre-defines the following named character entities:
+    text.gsub!(/\&amp;(amp;)/, "&\\1")
+    text.gsub!(/\&amp;([lg]t;)/, "&\\1")
+    text.gsub!(/\&amp;(quot;)/, "&\\1")
+    text.gsub!(/\&amp;(apos;)/, "&\\1")
 
     # From http://snippets.dzone.com/posts/show/1161
     str.unpack("U*").collect {|s| (s > 127 ? "&##{s};" : s.chr) }.join("")
-     
+
+    # Strip out control characters
+    text.gsub!(/[\x00-\x1f]/, '?')
+    text.gsub!(/\x7f/, '?')
 
     # Fix apostrophes so that they show up as expected. Fixes issue 26.
     text.gsub!('&#8217;', "'")
@@ -315,6 +322,8 @@ class Output
     text.gsub!(/\<img.*?\>/i, '[img]')
     text.gsub!(/\<.*?\>/m, ' *')
     text.gsub!(/\&nbsp\;/, ' ')
+    text.gsub!(/\&quot\;/, '"')
+    text.gsub!(/\&apos\;/, "'")
     text.gsub!(/\&rsquo\;/, "'")
     text.gsub!(/\&ndash;/, " - ")
     text.gsub!(/\&deg;/, "'")
@@ -333,7 +342,7 @@ class Output
   def generatePreOutput(title)
     output = replaceVariables(@outputFormat['templatePre'], nil)
   end
-  
+
   def updateShortNames()
     snames = {}
     @wpHash.each_key { |wid|
@@ -341,8 +350,8 @@ class Output
       if @waypointLength > 1
         shorter_name = shortName(cache['name'])
         shortest_name = shorter_name[0..(@waypointLength - 1)]
-        # If we have two caches that generate the same short name        
-        if snames.has_key?(shortest_name.upcase):
+        # If we have two caches that generate the same short name
+        if snames.has_key?(shortest_name.upcase)
           other_wid = snames[shortest_name.upcase]
           other_cache = @wpHash[other_wid]
           debug "Conflict found with #{shortest_name} (#{wid} vs #{other_wid})"
@@ -357,7 +366,7 @@ class Output
           shortest_name = shorter_name[0..(@waypointLength - 4)] + unique_chars[0..3]
           displayMessage "Resolved short-name conflict for #{wid} (#{shortest_name}) and #{other_wid} (#{other_cache['sname']})"
         end
-        
+
         snames[shortest_name.upcase] = wid
         cache['sname'] = shortest_name
         cache['snameUncut'] = shorter_name
@@ -369,30 +378,30 @@ class Output
       end
     }
   end
-  
+
   def generateHtmlIndex()
     # Returns an index and a hash of wid -> symbol list
     index = '<ul>'
     symbolHash = Hash.new
 
-    @wpHash.keys.sort.each { |wid| 
+    @wpHash.keys.sort.each { |wid|
       cache = @wpHash[wid]
       symbolHash[wid] = ''
       if (@wpHash[wid]['travelbug'])
         symbolHash[wid] = "<b><font color=\"#11CC11\">&euro;</font></b>"
       end
 
-      if (@wpHash[wid]['terrain'] > 3)
+      if (@wpHash[wid]['terrain'] >= 3.5)
         symbolHash[wid] << "<b><font color=\"#999922\">&sect;</font></b>"
       end
 
-      if (@wpHash[wid]['funfactor'] >= 3.0)
-        symbolHash[wid] << "<b><font color=\"#BB2222\">&hearts;</font></b>"
-      elsif (@wpHash[wid]['funfactor'] >= 2.1)
-        symbolHash[wid] << "<b><font color=\"#BB9999\">&hearts;</font></b>"
+      if @wpHash[wid]['funfactor']
+        if @wpHash[wid]['funfactor'] >= 3.5
+          symbolHash[wid] << "<b><font color=\"#BB6666\">&hearts;</font></b>"
+        end
       end
 
-      if (@wpHash[wid]['difficulty'] > 3)
+      if (@wpHash[wid]['difficulty'] >= 3)
         symbolHash[wid] << "<b><font color=\"#440000\">&uarr;</font></b>"
       end
 
@@ -404,7 +413,7 @@ class Output
       if @wpHash[wid]['mdays'] < 0
         index_item << '<strong>'
       end
-      
+
       index_item << "<a href=\"\##{wid}\">#{cache['name']}</a>"
       if @wpHash[wid]['mdays'] < 0
         index_item << '</strong>'
@@ -416,13 +425,13 @@ class Output
     index << '</ul>'
     return [index, symbolHash]
   end
-  
+
   def createGpxCommentLogs(cache)
     if not cache['comments']
       debug "No comments found for #{cache['name']}"
       return nil
     end
-    
+
     entries = []
     debug "Generating comment XML for #{cache['name']}"
     cache['comments'].each { |comment|
@@ -446,7 +455,10 @@ class Output
 
   def decryptHint(hint)
     if hint
-      return hint.tr('A-MN-Z', 'N-ZA-M').tr('a-mn-z', 'n-za-m')
+      decrypted = hint.tr('A-MN-Z', 'N-ZA-M').tr('a-mn-z', 'n-za-m')
+      # Oops, we don't need to decrypt the text within brackets - it's raw.
+      decrypted.gsub!(/(\[.*?\])/) { $1.tr('A-MN-Z', 'N-ZA-M').tr('a-mn-z', 'n-za-m') }
+      return decrypted
     else
       return ''
     end
@@ -463,9 +475,9 @@ class Output
     if cache['distance']
       relative_distance = cache['distance'].to_s + 'mi ' + cache['direction']
     else
-      relative_distance = ''
+      relative_distance = 'N/A'
     end
-    
+
     if get_location
       geocoder = GeoCode.new()
       location = geocoder.lookup_coords(cache['latdata'], cache['londata'])
@@ -474,7 +486,13 @@ class Output
     end
     coord_query = URI.escape("#{cache['latdata']},#{cache['londata']}")
     available = (not cache['disabled'])
-    
+
+    if @username and cache['visitors'].include?(@username)
+      symbol = 'Geocache Found'
+    else
+      symbol = 'Geocache'
+    end
+
     variables = {
       'wid' => wid,
       'symbols' => symbols,
@@ -493,9 +511,10 @@ class Output
       'relativedistance' => relative_distance,
       'hintdecrypt' => decryptHint(cache['hint']),
       'hint' => cache['hint'],
+      'cacheSymbol' => symbol,
     }
   end
-    
+
   def generateOutput(title)
     debug "generating output: #{@outputType} - #{$Format[@outputType]['desc']}"
     @outVars = Hash.new
@@ -512,7 +531,7 @@ class Output
     else
       symbolHash = nil
     end
-    
+
     counter = 0
     @wpHash.keys.sort.each { |wid|
       cache = @wpHash[wid]
